/***************************************************************************

                          colorsprite.h 
                          a sprite for SDL-Apps that consists just of
                          color, without any images/textures
                             -------------------
    begin                : Mon Mai 26 2003
    copyright            : (C) 2003 by Thomas Unterthiner
    email                : thomas_unterthiner@web.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef COLORSPRITE_H
#define COLORSPRITE_H

#include <SDL.h>
#include <vector>

class ColorSprite
{
public: 
	ColorSprite(int PosX, int PosY, int Width, int Height,
	            Uint32 Color,
	            Uint32 BackgroundColor = 0);
	// the normal Ctor, initiates all Members

	ColorSprite(const ColorSprite&);
	// copy-ctor
	// WATCH OUT: doesn't copy the redraw-information

	virtual ~ColorSprite();
	// Deconstructor, doesn't anything
 
	virtual void draw(SDL_Surface* Surface);
	// draws the Sprite on the surface, and - if neccessary - it cancels the old
	// image of the sprite from the surface
	// 	PARAMETER
	// 		SDL_Sprite* Surface: the Surface to draw into

	virtual void move(int ByX, int ByY);
	// moves the sprite by a certain amount of pixel.
 	// 	PARAMETER
 	// 		int ByX, ByY: move the sprite by so many X/Y Pixel

	void setPos(int PosX, int PosY);
	// moves the sprite to a certain position.
	// 	PARAMETER
	// 		int PosX, PosY: new Coordinates of the sprite.

	SDL_Rect& getCoordinates() {return mCoordinates;}
	const SDL_Rect& getCoordinates() const {return mCoordinates;}
	// returns a Reference to the coordinates

protected:
	SDL_Rect mCoordinates;
	Uint32 mColor;
	Uint32 mRedrawColor;
	bool mPositionChanged;
	SDL_Rect mRedrawCoordinates;

	void setSize(int Width, int Height);
	// sets a new Sprite-Size
	// 	PARAMETER
	// 		int PosX, PosY: new Coordinates of the sprite.

private:
	inline void redraw(SDL_Surface* Surface);
	// cancels the old images of the sprite from the surface, by
	// redrawing all older spritepositions
	// 	PARAMETER
 	// 		SDL_Sprite* Surface: the Surface to draw into

  void saveLastDrawnPosition();
  // saves the last drawn position of the sprite into mRedrawCoordinates
};

#endif
