/***************************************************************************
                          bluePong, (C) 2003
                          Thomas Unterthiner                         

                          ponggame.cpp
                          a simple game of bluePong
                             -------------------
    begin                : Sam Mai 31 2003
    copyright            : (C) 2003 by Thomas Unterthiner
    email                : thomas_unterthiner@web.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "ponggame.h"


// ---------------
// public methods
// ---------------

bool PongGame::nextTick()
/*****************************************************************************
 * makes the next game-movement: moves the paddles & the ball, handles the
 * input etc.
 * 	RETURNS
 * 		FALSE if the game is over, otherwise TRUE
 *****************************************************************************/
{
	if ( isGameOver() )
	{
		blinkBar( mPlayer[mLastPlayerWithBall] );
		return false;
	}

	for (unsigned short i = 0; i < mPlayer.size(); ++i)
	{
		handleInput(mPlayer[i]);
	}

	switch ( handleCollisionWithWall(*mBall) )
	{
		case ALIGN_VERTICAL:
			newPoint();
			if ( isGameOver() )
			{
				blinkBar( mPlayer[mLastPlayerWithBall] );
				return false;
			}
			else	// dont reset Ball when the game is over (would reset 
			{		// mLastPlayerWithBall, and then the blinkBar() wouldn't work)	                  
				resetBall(mBall); 
			}
			break;
		case ALIGN_HORIZONTAL:
			if (mPlayer.size() > 2)
			{
				newPoint();
				if ( isGameOver() )
				{
					blinkBar( mPlayer[mLastPlayerWithBall] );
					return false;
				}
				else	// dont reset Ball when the game is over (would reset 
				{		// mLastPlayerWithBall, and then the blinkBar() wouldn't work)
					resetBall(mBall);
				}
			}
			else
				ChangeBallDir(ALIGN_HORIZONTAL);
			break;
	}
	
	switch (handleCollisionWithPaddle(*mBall) )
	{
		case ALIGN_HORIZONTAL:
			ChangeBallDir(ALIGN_HORIZONTAL);
			break;
		case ALIGN_VERTICAL:
//			increaseBallSpeed(mBall);
			ChangeBallDir(ALIGN_VERTICAL);
			break;
		default:
			// no collison, don't do anything ;-)
			break;
	}

	mBall->move();
	drawSprites();
	return true;
}



// ------------------
// protected methods
// ------------------


short PongGame::handleCollisionWithWall(ColorSprite& Sprite)
/*****************************************************************************
 * checks if another Sprite collides with a wall
 * 	RETURNS
 * 	0 = no collision
 * 	otherwhise: Alignment of the wall
 *****************************************************************************/
{
	for (unsigned short i = 0; i<mWall.size(); ++i)
	{
		if( mWall[i]->isCollidingWith(Sprite) )
		{
			return mWall[i]->getAlignment();
		}
	}
	return 0;
}


short PongGame::handleCollisionWithPaddle(PongBall& Ball)
/*****************************************************************************
 * checks if the Ball collides with a paddle
 * 	RETURNS
 * 	0 = no collision
 * 	otherwhise: Alignment of the paddle
 *****************************************************************************/
{
	for (int i = 0; i<mPlayer.size(); ++i)
	{
		if( mPlayer[i]->getPaddle().isCollidingWith(Ball) )
		{
			mLastPlayerWithBall = i;
			return mPlayer[i]->getPaddle().getAlignment();
		}
	}
	return 0;
}


 
void PongGame::ChangeBallDir(Alignment CollisionObjectAlignment)
/*****************************************************************************
 * changes the Direction of the Ball
 * 	 Alignment CollisionObjectAlignment: Alignment of the sprite the ball
 * 	                                     collided with
 *****************************************************************************/
{
	switch (CollisionObjectAlignment)
	{
		case ALIGN_HORIZONTAL:
			mBall->setSpeedY( mBall->getSpeedY() * -1);
			break;
		case ALIGN_VERTICAL:
			mBall->setSpeedX( mBall->getSpeedX() * -1);
			break;
	}
}



bool PongGame::isGameOver()
/*****************************************************************************
 * checks if the game is over, meaning if someone reached mMaxPoints
 * 	RETURNS
 * 		TRUE if game is over, otherwise FALSE
 *****************************************************************************/
{
	for (unsigned short i = 0; i < mPlayer.size(); ++i)
	{
		if (mPlayer[i]->getPoints() == mMaxPoints)
		{
			return true;
		}
	}
	return false;
}



void PongGame::handleInput(PongPlayer* Player)
/*****************************************************************************
 * handles the input (moves the paddles) for a specific player
 * if the paddle of a player is aligned vertically, only the up/down
 * movements will be checked... if it's aligned horizontally, only right/left
 * keyevents will be controlled
 *****************************************************************************/
{

	Uint8* KeyStatus = SDL_GetKeyState(NULL);	// get pressed keys

   if ( Player->getPaddle().getAlignment() == ALIGN_VERTICAL )
   {
		if ( KeyStatus[Player->getKey(TOP_SIDE)] )
		{
			if (! mWall[TOP_SIDE]->isCollidingWith(Player->getPaddle() ) )
				Player->getPaddle().move(-BALL_SPEED);
		}

		if ( KeyStatus[Player->getKey(BOTTOM_SIDE)] )
		{
			if (! mWall[BOTTOM_SIDE]->isCollidingWith(Player->getPaddle() ) )
				Player->getPaddle().move(BALL_SPEED);
		}
	}
	else
	{
		if ( KeyStatus[Player->getKey(LEFT_SIDE)] )
		{
			if (! mWall[LEFT_SIDE]->isCollidingWith(Player->getPaddle() ) )
				Player->getPaddle().move(-BALL_SPEED);
		}

		if ( KeyStatus[Player->getKey(RIGHT_SIDE)] )
		{
			if (! mWall[RIGHT_SIDE]->isCollidingWith(Player->getPaddle() ) )
				Player->getPaddle().move(BALL_SPEED);
		}
	}
}



void PongGame::newPoint()
/*****************************************************************************
 * assign a point to the last player who interacted with the ball,
 * then draw the actual point-status in a bar behind the player's wall
 *****************************************************************************/
{
	if ( mLastPlayerWithBall != -1) // make sure somebody already touched the ball
	{
		mPlayer[mLastPlayerWithBall]->addPoint(1);
		drawPlayerPoints( mPlayer[mLastPlayerWithBall] );
	}
/*
	else
	{
		mPlayer[!mLastPlayerWithBall]->addPoint(-1); // lose Point
		drawPlayerPoints( mPlayer[!mLastPlayerWithBall] );
	}
*/
}



void PongGame::drawSprites()
/*****************************************************************************
 * draws all the sprites
 *****************************************************************************/
{
	for (int i = 0; i < mPaddle.size(); ++i)
	{
		mPaddle[i]->draw(mDisplay);
	}

	mBall->draw(mDisplay);

	for (int j = 0; j < mWall.size(); ++j)
	{
		mWall[j]->draw(mDisplay);
	}
}



void PongGame::resetBall(PongBall* Ball)
/*****************************************************************************
 * resets the ball's position and speed
 *****************************************************************************/
{
	Ball->setPos( mWall[2]->getCoordinates().x + (mWall[2]->getCoordinates().w / 2),
	              mWall[0]->getCoordinates().y + (mWall[0]->getCoordinates().h / 2) );

	Ball->setSpeedX(BALL_SPEED);
	Ball->setSpeedY(BALL_SPEED);
	mLastPlayerWithBall = -1;
}



void PongGame::increaseBallSpeed(PongBall* Ball, int By)
/*****************************************************************************
 * simply increases the speed of the ball...
 * NOTICE: if ball speed is negative, obviously the moving-speed of the ball
 *         has to be decreased
 * 	PARAMETER
 * 		int By = increase-speed in pixel/tick (1 per default)
 *****************************************************************************/
{
	// x - speed
	if (Ball->getSpeedX() >= 0 )
		Ball->setSpeedX( Ball->getSpeedX() + By);
	else
		Ball->setSpeedX( Ball->getSpeedX() - By);

	// y-speed
	if (Ball->getSpeedY() >= 0 )
		Ball->setSpeedY( Ball->getSpeedY() + By);
	else
		Ball->setSpeedY( Ball->getSpeedY() - By);
}




void PongGame::drawPlayerPoints(PongPlayer* Player)
/*****************************************************************************

 *****************************************************************************/
{
	fillPointSideBar( Player, Player->getColor() );
}



void PongGame::blinkBar(PongPlayer* Player)
/*****************************************************************************
 * lets the point-bar of the player blink (function gets called once we have
 * a winner)
 *****************************************************************************/
{
	static Uint32 BlinkColor;
	
	BlinkColor = (BlinkColor == 0) ? Player->getColor()
	                               : 0;

	fillPointSideBar( Player, BlinkColor );
	SDL_Delay(333);
}



void PongGame::fillPointSideBar(PongPlayer* Player, Uint32 Color)
/*****************************************************************************
 * gets the coordinates of the sidebar behind the wall of a player
 * that is used to display the scores, and fills that part of the Display
 * with a given color
 * on horizontal-aligned players, the bar increases from left to right,
 * on vertical-aligned players, the bar grows from bottom to top
 *****************************************************************************/
{
	PlayerSide Side = Player->getSide();
	SDL_Rect FillRect;
	unsigned int PixelPerPoint = 0;
	SDL_Rect WallPosition  = mWall[Side]->getCoordinates();
	
	// set some elments which are the same
	// for players with the same alignment
	if (Side  == LEFT_SIDE || Side == RIGHT_SIDE) // vertical player
	{
		PixelPerPoint =  WallPosition.h / mMaxPoints;
		FillRect.w = DISTANCE_WALL_FROM_VERTICAL_BORDER / 2;
		FillRect.h = Player->getPoints() * PixelPerPoint;
	}
	else // horizontal player
	{
		PixelPerPoint =  WallPosition.w / mMaxPoints;
		FillRect.w = Player->getPoints() * PixelPerPoint;
		FillRect.h = DISTANCE_WALL_FROM_HORIZONTAL_BORDER / 2;
	}

	// set the remaining elements
	switch (Side)
	{
		case LEFT_SIDE:
			FillRect.x = WallPosition.x
							- DISTANCE_WALL_FROM_VERTICAL_BORDER
							+ (DISTANCE_WALL_FROM_VERTICAL_BORDER / 4);
			FillRect.y = WallPosition.y + (mMaxPoints - Player->getPoints() ) * PixelPerPoint;
			break;
		case RIGHT_SIDE:
			FillRect.x = WallPosition.x + (DISTANCE_WALL_FROM_VERTICAL_BORDER / 4);
			FillRect.y = WallPosition.y + (mMaxPoints - Player->getPoints() ) * PixelPerPoint;
			break;
		case TOP_SIDE:
			FillRect.x = WallPosition.x;
			FillRect.y = WallPosition.y - 3*(DISTANCE_WALL_FROM_HORIZONTAL_BORDER / 4);
			break;
		case BOTTOM_SIDE:
			FillRect.x = WallPosition.x;
			FillRect.y = WallPosition.y + (DISTANCE_WALL_FROM_HORIZONTAL_BORDER / 4);
			break;
	}

	SDL_FillRect(mDisplay, &FillRect, Color);
	SDL_UpdateRect(mDisplay, FillRect.x, FillRect.y, FillRect.w, FillRect.h);
}

// ---------------------
//  Ctor/Dtor and
//  Ctor help-functions
// ---------------------


PongGame::PongGame(SDL_Surface* Display)
/*****************************************************************************
 * the normal Ctor
 *****************************************************************************/
         : mDisplay(Display),
           mLastPlayerWithBall(-1)
{
	setConfigDefaults();
	createWalls();
	createPaddles(2);
	createPlayers(2);

	mBall = new PongBall( 0,  // doesn't matter where the ball is placed,
	                      0,  // it will be replaced by resetBall() anyway
	                      BALL_WIDTH,
	                      BALL_HEIGHT,
	                      SDL_MapRGB(mDisplay->format, 0, 255, 0),	// green ball
                          0,
	                      1,
	                      1);
	resetBall(mBall);
}



void PongGame::createWalls()
/*****************************************************************************
 * creates the walls for the game... used by the Ctor only
 * NOTE: the walls MUST be created in the order defined by enum PlayerSide
 *       which is declared in PongPlayer.h
 *****************************************************************************/
{
	SDL_Rect Display;
	SDL_GetClipRect(mDisplay, &Display);
	
	Uint32 WallColor = SDL_MapRGB(mDisplay->format, 255, 255, 255);

	// right wall
	mWall.push_back( new PongWall( Display.w - DISTANCE_WALL_FROM_VERTICAL_BORDER,
	                               Display.y + DISTANCE_WALL_FROM_HORIZONTAL_BORDER,
	                               WALL_THICKNESS,
	                               Display.h - (DISTANCE_WALL_FROM_HORIZONTAL_BORDER * 2),
	                               WallColor,
	                               ALIGN_VERTICAL) );
	// left wall
	mWall.push_back( new PongWall( Display.x + DISTANCE_WALL_FROM_VERTICAL_BORDER ,
	                               Display.y + DISTANCE_WALL_FROM_HORIZONTAL_BORDER,
	                               WALL_THICKNESS,
	                               Display.h - (DISTANCE_WALL_FROM_HORIZONTAL_BORDER * 2),
	                               WallColor,
	                               ALIGN_VERTICAL) );
	// top wall
	mWall.push_back( new PongWall( Display.x + DISTANCE_WALL_FROM_VERTICAL_BORDER,
	                               Display.y + DISTANCE_WALL_FROM_HORIZONTAL_BORDER,
	                               Display.w - (DISTANCE_WALL_FROM_VERTICAL_BORDER * 2),
	                               WALL_THICKNESS,
	                               WallColor,
	                               ALIGN_HORIZONTAL) );
	// bottom wall
	mWall.push_back( new PongWall( Display.x + DISTANCE_WALL_FROM_VERTICAL_BORDER,
	                               Display.h - DISTANCE_WALL_FROM_HORIZONTAL_BORDER - 1,
	                               Display.w - (DISTANCE_WALL_FROM_VERTICAL_BORDER * 2),
	                               WALL_THICKNESS,
	                               WallColor,
	                               ALIGN_HORIZONTAL) );
}



void PongGame::createPlayers(short PlayerCount)
/*****************************************************************************
 * creates the players for the game... used by the Ctor only
 *	PARAMETER
 *		short PlayerCount: amount of players to be created
 *****************************************************************************/
{
	SDLKey Player1Keys[4] = {SDLK_RIGHT, SDLK_RIGHT, SDLK_a, SDLK_z};
	SDLKey Player2Keys[4] = {SDLK_RIGHT, SDLK_RIGHT, SDLK_UP, SDLK_DOWN};


	// NOTE: in the following switch-case - statement, there are
	//       no "break"-statements after the case's. I DID THAT ON
	//       PURPOSE!!!
	//       This way, the control-flow jumps in on the needed amount
	//       of players and creates the correct number of players
	//       right ahead.
	switch (PlayerCount)
	{
		case 4:	mPlayer.push_back( new PongPlayer (mPaddle[BOTTOM_SIDE]->getColor(),
				                                   "Player 4",
						                           Player2Keys,
									               sizeof(Player2Keys)/sizeof(SDLKey),
												   *mPaddle[BOTTOM_SIDE],
												   BOTTOM_SIDE) );
		
		case 3:	mPlayer.push_back( new PongPlayer (mPaddle[TOP_SIDE]->getColor(),
				                                   "Player 3",
							                       Player2Keys,
										           sizeof(Player2Keys)/sizeof(SDLKey),
				                                   *mPaddle[TOP_SIDE],
						                           TOP_SIDE) );

		case 2: mPlayer.push_back( new PongPlayer (mPaddle[RIGHT_SIDE]->getColor(),
									               "Player 2",
													Player2Keys,
													sizeof(Player2Keys)/sizeof(SDLKey),
													*mPaddle[RIGHT_SIDE],
													RIGHT_SIDE) );

	
		case 1: mPlayer.push_back( new PongPlayer (mPaddle[LEFT_SIDE]->getColor(),
							                       "Player 1",
									               Player1Keys,
										           sizeof(Player1Keys)/sizeof(SDLKey),
											       *mPaddle[LEFT_SIDE],
												   LEFT_SIDE) );
	}
}



void PongGame::createPaddles(short PaddleCount)
/*****************************************************************************
 * creates the paddles for the game... used by the Ctor only
 *	PARAMETER
 *		short PaddleCount: amount of paddles to be created
 *****************************************************************************/
{
	// NOTE: in the following switch-case - statement, there are
	//       no "break"-statements after the case's. I DID THAT ON
	//       PURPOSE!!!
	//       This way, the control-flow jumps in on the needed amount
	//       of players and creates the correct number of players
	//       right ahead.
	switch (PaddleCount)
	{
	// horzontal paddles
	// =================
	case 4:	mPaddle.push_back ( new PongPaddle (mWall[BOTTOM_SIDE]->getCoordinates().x           // center
	                                            	+ (mWall[BOTTOM_SIDE]->getCoordinates().w/2) // paddle
	                                            	- (PADDLE_WIDTH/2),                          // horizontally
					                            mWall[BOTTOM_SIDE]->getCoordinates().y // enough distance
	                                            	- PADDLE_WIDTH                     // between wall and
	                                            	- BALL_HEIGHT*2,                   // paddle for a ball
	                                            PADDLE_HEIGHT,
	                                            PADDLE_WIDTH,
	                                            SDL_MapRGB(mDisplay->format, 255, 255, 0),
	                                            ALIGN_HORIZONTAL) );

	case 3: mPaddle.push_back ( new PongPaddle (mWall[TOP_SIDE]->getCoordinates().x
	                                            	+ (mWall[TOP_SIDE]->getCoordinates().w/2)
	                                            	- (PADDLE_WIDTH/2),
	                                            mWall[TOP_SIDE]->getCoordinates().y
	                                            	+ PADDLE_WIDTH
	                                            	+ BALL_HEIGHT*2,
	                                            PADDLE_HEIGHT,
	                                            PADDLE_WIDTH,
	                                            SDL_MapRGB(mDisplay->format, 255, 0, 255),
	                                            ALIGN_HORIZONTAL) );
		
	// vertical paddles
	// ================
	case 2: mPaddle.push_back ( new PongPaddle (mWall[RIGHT_SIDE]->getCoordinates().x // enough distance
	                                            	- PADDLE_WIDTH                    // between wall and
	                                            	- BALL_WIDTH*2,                   // paddle for a ball
	                                            mWall[RIGHT_SIDE]->getCoordinates().y           // center
	                                            	+ (mWall[RIGHT_SIDE]->getCoordinates().h/2) // paddle
	                                            	- (PADDLE_HEIGHT/2),                        // vertically
	                                            PADDLE_WIDTH,
	                                            PADDLE_HEIGHT,
	                                            SDL_MapRGB(mDisplay->format, 0, 0, 255),
	                                            ALIGN_VERTICAL) );

	case 1: mPaddle.push_back ( new PongPaddle (mWall[LEFT_SIDE]->getCoordinates().x
	                                            	+ PADDLE_WIDTH
	                                            	+ BALL_WIDTH*2,
	                                            mWall[LEFT_SIDE]->getCoordinates().y
	                                            	+ (mWall[LEFT_SIDE]->getCoordinates().h/2)
	                                            	- (PADDLE_HEIGHT/2),
	                                            PADDLE_WIDTH,
	                                            PADDLE_HEIGHT,
	                                            SDL_MapRGB(mDisplay->format, 255, 0, 0),
	                                            ALIGN_VERTICAL) );
	}
}


PongGame::~PongGame()
/*****************************************************************************
 * the normal Dtor
 *****************************************************************************/
{
	delete mBall;
	
	for (int i = 0; i < mPaddle.size(); ++i)
	{
		delete mPaddle[i];
	}
	
	for (int j = 0; j < mPlayer.size(); ++j)
	{
		delete mPlayer[j];
	}

	for (int k = 0; k < mWall.size(); ++k)
	{
		delete mWall[k];
	}
}

void PongGame::setConfigDefaults()
/*****************************************************************************
 * sets all config-data to default-values
 *****************************************************************************/
{
	DISTANCE_WALL_FROM_HORIZONTAL_BORDER = 50;
	DISTANCE_WALL_FROM_VERTICAL_BORDER = 50;
	WALL_THICKNESS = 1;
	PADDLE_WIDTH = 7;
	PADDLE_HEIGHT = 75;
	BALL_WIDTH = 10;
	BALL_HEIGHT = 10;
	mMaxPoints = 3;
	BALL_SPEED = 2;
}
