/***************************************************************************

                          pongwall.cpp
                          a wall - a special sprite - in a game of
                          bluePong
                             -------------------
    begin                : Mon Mai 26 2003
    copyright            : (C) 2003 by Thomas Unterthiner
    email                : thomas_unterthiner@web.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "pongwall.h"

PongWall::PongWall(int PosX, int PosY, int Width, int Height,
                   Uint32 Color,
                   Alignment Align,
                   Uint32 BackgroundColor)
/*****************************************************************************
 * the normal Ctor
 *****************************************************************************/
         : ColorSprite(PosX, PosY, Width, Height, Color, BackgroundColor),
           mAlignment(Align)
{
	
}




bool PongWall::isCollidingWith(const ColorSprite& OtherSprite) const
/*****************************************************************************
 * checks if this object collides with another ColorSprite
 *
 * COLLISION-ALGORYTHM:
 * 	Note: x gets bigger going to the left
 * 	      y gets bigger going down (NOT like in mathematical Coordinates!!!)
 * 	Ax = most left point of Sprite A
 * 	Bx = most left point of Sprite B
 * 	Ay && By = topmost points of the sprites
 * 	Ax+Aw = most right point of Sprite A
 * 	Bx+Bw = most right point of Sprite B
 * 	Ay+Ah && By+Bh = bottommost points of the sprites
 * 		Ax+Aw => Bx && Ax =< Bx+Bw ==> SpriteA and B are on the same width
 * 		Ay+Ah >= By && Ay =< By+Bh ==> They're on the same height
 * 	PARAMETER
 * 		const ColorSprite& b: the other sprite
 * 	RETURNS
 * 		TRUE on collision, FALSE in any other case
 *****************************************************************************/
{
	const SDL_Rect& A = this->getCoordinates();
	const SDL_Rect& B = OtherSprite.getCoordinates();

	// possible x-collision
	if ( ( A.x + A.w >= B.x)
		&& (A.x <= (B.x + B.w) ) )
	{
		// possible y-collision
		if ( ( (A.y+A.h)  >= B.y)
		&& (A.y <= B.y + B.h ) )
		{
			return true;
		}
	}
	return false;
}
